#include "chad.h"

CChadGame::CChadGame() {
	m_Ogre = new Root; 
	this->loadConfig();

	if(!m_Ogre->showConfigDialog()) return;

	m_Ogre->initialise(true, "Hanging Chad"); 
	this->initialise();
	this->createOutdoorScene();
}

CChadGame::~CChadGame() {
	delete m_EventProcessor;
	delete m_Player;
	delete m_Ogre;
}

bool CChadGame::loadConfig() {
	ConfigFile cf;
	cf.load("resources.cfg");

	ConfigFile::SectionIterator seci = cf.getSectionIterator();
	String secName, typeName, archName;

	while (seci.hasMoreElements()) {
		secName = seci.peekNextKey();
		ConfigFile::SettingsMultiMap *settings = seci.getNext();
		ConfigFile::SettingsMultiMap::iterator i;

		for (i = settings->begin(); i != settings->end(); ++i) {
			typeName = i->first;
			archName = i->second;
			ResourceGroupManager::getSingleton().addResourceLocation(archName, typeName, secName);
		}
	}

	return true;
}


void CChadGame::initialise() {
	m_EventProcessor = new EventProcessor(); 
	m_EventProcessor->initialise(m_Ogre->getAutoCreatedWindow()); 
	m_EventProcessor->startProcessingEvents(); 

	m_InputReader = m_EventProcessor->getInputReader();
	
	m_Ogre->addFrameListener(this);

	m_EventProcessor->addMouseListener(this);
	m_EventProcessor->addMouseMotionListener(this);
	m_EventProcessor->addKeyListener(this);

	ResourceGroupManager::getSingleton().initialiseAllResourceGroups();

	m_SceneMgr = NULL;
}

int CChadGame::run() {
	m_Player = new CChadPlayer();

	m_Ogre->startRendering();
	return 0;
}

void CChadGame::createOutdoorScene() {
	ResourceGroupManager::getSingleton().clearResourceGroup(ResourceGroupManager::getSingleton().getWorldResourceGroupName());
	ResourceGroupManager::getSingleton().initialiseAllResourceGroups();

	scenemanager = ST_EXTERIOR_CLOSE;

	if (m_SceneMgr != NULL) {
		m_Ogre->getAutoCreatedWindow()->removeAllViewports();
		m_SceneMgr->destroyAllCameras();
	}

	m_SceneMgr = m_Ogre->createSceneManager(ST_EXTERIOR_CLOSE);
	m_SceneMgr->setAmbientLight( ColourValue(0.5, 0.5, 0.5) );
	m_SceneMgr->setShadowTechnique(SHADOWTYPE_STENCIL_ADDITIVE);

	m_SunY = 180.0;
	m_SunZ = 0.0;

	m_Camera = m_SceneMgr->createCamera("Player Cam"); 

	m_Viewport = m_Ogre->getAutoCreatedWindow()->addViewport(m_Camera);

	m_SceneMgr->setWorldGeometry("terrain.cfg");

	m_Ogre->getAutoCreatedWindow()->removeAllViewports();
	m_SceneMgr->destroyAllCameras();

	m_SceneMgr = m_Ogre->createSceneManager(ST_EXTERIOR_CLOSE);
	m_SceneMgr->setAmbientLight( ColourValue(0.35, 0.35, 0.35) );
	m_SceneMgr->setShadowTechnique(SHADOWTYPE_STENCIL_ADDITIVE);

	m_SunLight = m_SceneMgr->createLight("Sun");
	m_SunLight->setType(Light::LT_DIRECTIONAL);
	m_SunLight->setDiffuseColour(1.0, 1.0, 1.0);
	m_SunLight->setSpecularColour(1.0, 1.0, 1.0);
	m_SunLight->setDirection(Vector3(0, sin(m_SunY), sin(m_SunZ)));

	m_MoonLight = m_SceneMgr->createLight("Moon");
	m_MoonLight->setType(Light::LT_DIRECTIONAL);
	m_MoonLight->setDiffuseColour(0.3, 0.3, 0.5);
	m_MoonLight->setSpecularColour(0.0, 0.0, 0.0);
	m_MoonLight->setDirection(Vector3(0, -sin(m_SunY), sin(m_SunZ)));

	m_SceneMgr->setShowDebugShadows(true);

	m_Camera = m_SceneMgr->createCamera("Player Cam"); 
	m_Camera->setPosition(Vector3(750,70,350));
	m_Camera->setNearClipDistance(1);
	m_Camera->setDirection(90, 10, 0);

	m_Viewport = m_Ogre->getAutoCreatedWindow()->addViewport(m_Camera);

	m_SceneMgr->setWorldGeometry("terrain.cfg");
	m_SceneMgr->setSkyDome(true, "Examples/CloudySky");

	Entity* OceanEntity;
	Plane OceanPlane;

	OceanPlane.normal = Vector3::UNIT_Y;
	MeshManager::getSingleton().createPlane(
		"OceanPlane",
		ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME,
		OceanPlane,
		16000, 16000,
		10, 10,
		true, 1,
		50, 50,
		Vector3::UNIT_Z);

	OceanEntity = m_SceneMgr->createEntity("water", "OceanPlane");
	OceanEntity->setMaterialName("Chad/Water");
	OceanNode = m_SceneMgr->getRootSceneNode()->createChildSceneNode("OceanNode");
	OceanNode->attachObject(OceanEntity);
	OceanNode->translate(2000, 20, 0);

	HouseEntity = m_SceneMgr->createEntity("house", "tudorhouse.mesh");
	SceneNode* HouseNode = m_SceneMgr->getRootSceneNode()->createChildSceneNode("HouseNode");
	HouseNode->attachObject(HouseEntity);
	HouseNode->translate(1100,137, 500);
	HouseNode->scale(0.25,0.125,0.25);
}

void CChadGame::createIndoorScene() {
	scenemanager = ST_INTERIOR;

	if (m_SceneMgr != NULL) {
		m_Ogre->getAutoCreatedWindow()->removeAllViewports();
		m_SceneMgr->destroyAllCameras();
	}

	m_SceneMgr = m_Ogre->createSceneManager("BspSceneManager");

	ResourceGroupManager::getSingleton().linkWorldGeometryToResourceGroup(
		ResourceGroupManager::getSingleton().getWorldResourceGroupName(), 
		"maps/chiropteradm.bsp", m_SceneMgr);

	ResourceGroupManager::getSingleton().initialiseAllResourceGroups();

	ResourceGroupManager::getSingleton().clearResourceGroup(ResourceGroupManager::getSingleton().getWorldResourceGroupName());
	ResourceGroupManager::getSingleton().loadResourceGroup(ResourceGroupManager::getSingleton().getWorldResourceGroupName(), false, true);

	m_Camera = m_SceneMgr->createCamera("Player Cam"); 
	m_Camera->setNearClipDistance(4);
	m_Camera->setFarClipDistance(4000);

	m_Viewport = m_Ogre->getAutoCreatedWindow()->addViewport(m_Camera);
	m_Viewport->setBackgroundColour(ColourValue(0, 0, 0));

	ViewPoint vp = m_SceneMgr->getSuggestedViewpoint(true);
	m_Camera->setPosition(vp.position);
	m_Camera->pitch(Degree(90));
	m_Camera->rotate(vp.orientation);
	m_Camera->setFixedYawAxis(true, Vector3::UNIT_Z);
}

bool CChadGame::frameStarted(const FrameEvent& evt) {
	switch (scenemanager) {
		case ST_EXTERIOR_CLOSE:
			return frameStartedOutside();
			break;

		case ST_INTERIOR:
			return frameStartedInside();
			break;
	}

	return true;
}

bool CChadGame::frameStartedOutside() {
	m_SunLight->setDirection(Vector3(0, sin(m_SunY), sin(m_SunZ)));
	m_MoonLight->setDirection(Vector3(0, -sin(m_SunY), -sin(m_SunZ)));

	m_SunY += 0.001;
	m_SunZ += 0.001;

	m_InputReader->capture(); 

	if(m_InputReader->isKeyDown(KC_ESCAPE)) return false; 

	Vector3 translateVector = Vector3::ZERO;

	float playerspeed = m_Player->getSpeed();

	if (m_InputReader->isKeyDown(KC_W)) translateVector.z = -playerspeed;
	if (m_InputReader->isKeyDown(KC_S)) translateVector.z = +playerspeed;
	if (m_InputReader->isKeyDown(KC_A)) translateVector.x = -playerspeed;
	if (m_InputReader->isKeyDown(KC_D)) translateVector.x = +playerspeed;
	if (m_InputReader->isKeyDown(KC_F1)) createIndoorScene();
	if (m_InputReader->isKeyDown(KC_F2)) createOutdoorScene();

	m_Camera->moveRelative(translateVector);

	Vector3 campos = m_Camera->getPosition();
	Ray camray(Vector3(campos.x, 1000.0f, campos.z), Vector3::NEGATIVE_UNIT_Y);

	RaySceneQuery* query = m_SceneMgr->createRayQuery( Ray() );

       	query->setRay(camray);

	RaySceneQueryResult &result = query->execute();
        RaySceneQueryResult::iterator iter = result.begin();

	if (iter != result.end() && iter->worldFragment) {
		float terrainheight = iter->worldFragment->singleIntersection.y;
		if ((terrainheight + 6.0f) != campos.y) m_Camera->setPosition(campos.x, terrainheight + 6.0f, campos.z);
        }

	m_SceneMgr->destroyQuery(query);

	OceanNode->translate(0, sin(m_OceanFlowNum) / 2000, 0);
	m_OceanFlowNum += 0.001;

	return true;
}

bool CChadGame::frameStartedInside() {
	m_InputReader->capture(); 

	if(m_InputReader->isKeyDown(KC_ESCAPE)) return false; 

	Vector3 translateVector = Vector3::ZERO;

	float playerspeed = m_Player->getSpeed();

	if (m_InputReader->isKeyDown(KC_W)) translateVector.z = -playerspeed * 5;
	if (m_InputReader->isKeyDown(KC_S)) translateVector.z = +playerspeed * 5;
	if (m_InputReader->isKeyDown(KC_A)) translateVector.x = -playerspeed * 5;
	if (m_InputReader->isKeyDown(KC_D)) translateVector.x = +playerspeed * 5;
	if (m_InputReader->isKeyDown(KC_F1)) createIndoorScene();
	if (m_InputReader->isKeyDown(KC_F2)) createOutdoorScene();

	m_Camera->moveRelative(translateVector);

	return true;
}

void CChadGame::keyPressed(KeyEvent *e) {
	switch (e->getKey()) {
		case KC_LCONTROL:
			m_Player->m_IsCrouching = true;
			break;
		case KC_LSHIFT:
			m_Player->m_IsRunning = true;
			break;
	}
}

void CChadGame::keyReleased(KeyEvent *e) {
	switch (e->getKey()) {
		case KC_LCONTROL:
			m_Player->m_IsCrouching = false;
			break;
		case KC_LSHIFT:
			m_Player->m_IsRunning = false;
			break;
	}
}

int main(int argc, char **argv) { 
	CChadGame* game = new CChadGame();
	game->run();
	delete game;

	return 0;
}
